! Copyright (C) 2025 Aleksander "olus2000" Sabak.
! See https://factorcode.org/license.txt for BSD license.
USING: accessors arrays assocs concurrency.combinators http http.client http.json kernel math namespaces
sequences words ;
IN: esolang_games.code_guess.070


CONSTANT: game-url "https://codeguessing.gay/extra/game70"


! Game

CONSTANT: grid-width 15
CONSTANT: grid-height 5


: find-in-grid ( grid char -- coords? )
  swap index [ grid-width 1 + /mod swap 2array ] [ f ] if* ;


TUPLE: state player goal bomb seq score ;

USE: io

: <state> ( json -- state )
  [ "grid" of
    [ CHAR: @ find-in-grid ]
    [ CHAR: + find-in-grid ]
    [ CHAR: * find-in-grid ] tri ]
  [ "s" of ] [ "score" of ] tri state boa ;


USE: prettyprint


: bomb?-swap ( horisontal vertical state -- moves moves )
  dup bomb>>
  [ [ bomb>> ] [ player>> ] [ goal>> ] tri [ first2 ] tri@
    drop nipd -rot [ = ] 2bi@ or [ swap ] when ] [ drop ] if ;


: prepare-moves ( state -- moves-first moves-second )
  dup [ player>> ] [ goal>> ] bi [ first2 ] bi@ swapd [ - ] 2bi@
  [ [ abs ] [ 0 > ] bi "left" "right" ? <array> ]
  [ [ abs ] [ 0 > ] bi "up" "down" ? <array> ] bi*
  rot bomb?-swap ;


! Networking

SYMBOL: game-session


: +session-cookie ( request -- request' )
  game-session get "session" <cookie> put-cookie ;


: current-state ( -- state )
  game-url <get-request> +session-cookie
  http-request-json nip <state> ;


: move ( dir -- state )
  game-url "POST" <json-request>
  swap 'H{ { "dir" _ } } <json-data> >>data
  +session-cookie http-request-json nip <state> ;


: naive-step ( -- )
  current-state prepare-moves append [ move drop ] each ;


: async-step ( -- )
  current-state prepare-moves append
  [ move drop ] parallel-each ;


: parallel-moves ( moves -- state )
  [ move ] parallel-map [ seq>> ] maximum-by ;


: sequential-step ( state -- state )
  prepare-moves append parallel-moves ;


: n-steps ( n -- state )
  current-state swap [ sequential-step ] times ;


: sequential-bomb-avoiding-step ( state -- state )
  prepare-moves
  [ [ f ] [ parallel-moves ] if-empty ] bi@ swap or ;


: n-bomb-avoiding-steps ( n -- state )
  current-state swap
  [ sequential-bomb-avoiding-step ] times ;


: avoid-bombs-until ( n -- state )
  current-state swap '[ dup score>> _ < ]
  [ sequential-bomb-avoiding-step ] while ;
